{$N+}
unit cmplx;

interface

type
        float = Double;
        Complex_Type = String[2 * SizeOf(Float)];
        Cmplx_Type = record
                leng: Byte;
                r, i: Float;
        end;

function Re(z: Complex_Type): Float;
function Im(z: Complex_Type): Float;
function Complex(x, y: Float): Complex_Type;
function Complex_to_Str(z: Complex_Type; d: ShortInt): String;
function CSum(x, y: Complex_Type): Complex_Type;
function CDiff(x, y: Complex_Type): Complex_Type;
function CProd(x, y: Complex_Type): Complex_Type;
procedure Cpolar_form(z: Complex_Type; var r, theta: Float);
procedure CPrint(z: Complex_Type; n: Word);

implementation

function Complex(x, y: Float): Complex_Type;
var
        result: Cmplx_Type;
begin
        with result do
        begin
                leng := 2 * SizeOf(Float);
                r := x;
                i := y;
        end;
        Complex := Complex_Type(result);
end;

function Complex_to_Str(z: Complex_Type; d: ShortInt): String;
var
        result1, result2: String;
begin
        if d > 0 then
        begin
                Str(Re(z):0:d, result1);
                Str(Im(z):0:d, result2);
        end
        else
        begin
                Str(Re(z):(d + 6), result1);
                Str(Im(z):(d + 6), result2);
        end;
        if Im(z) < 0 then
                Complex_to_Str := result1 + result2 + 'i'
        else
                Complex_to_Str := result1 + '+' + result2 + 'i';
end;

function Re(z: Complex_Type): Float;
begin
        Re := Cmplx_Type(z).r;
end;

function Im(z: Complex_Type): Float;
begin
        Im := Cmplx_Type(z).i;
end;


function CSum(x, y: Complex_Type): Complex_Type;
begin
        CSum := Complex(Re(x) + Re(y), Im(x) + Im(y));
end;

function CDiff(x, y: Complex_Type): Complex_Type;
begin
        CDiff := Complex(Re(x) - Re(y), Im(x) - Im(y));
end;

function CProd(x, y: Complex_Type): Complex_Type;
begin
        CProd := Complex(
                Re(x) * Re(y) - Im(x) * Im(y),
                Re(x) * Im(y) + Im(x) * Re(y)
        );
end;


function CAbs(z: Complex_Type): Float;
begin
        CAbs := Sqrt(Sqr(Re(z)) + Sqr(Im(z)));
end;

procedure CPolar_form(z: Complex_Type; var r, theta: Float);
begin
        r := CAbs(z);
        if Re(z) > 0 then
                theta := ArcTan(Im(z) / Re(z))
        else
                if Re(z) < 0 then
                        theta := Pi + ArcTan(Im(z) / Re(z))
                else
                        if Im(z) >= 0 then
                                theta := 0.5 * Pi
                        else
                                theta := - 0.5 * Pi
end;

procedure CPrint(z: Complex_Type; n: Word);
begin
        if Re(z) < 0.0 then
                Write(' - ', Abs(Re(z)):(n + 2):n)
        else
                Write(' + ', Re(z):(n + 2):n);
        if Im(z) < 0.0 then
                Write(' - ', Abs(Im(z)):(n + 2):n)
        else
                Write(' + ', Im(z):(n + 2):n);
        Write('*i');
        WriteLn;
end;

begin
end.
