constructor rational_matrix_obj.init(M0: rational_matrix);
var
        i, j: Word;
begin
        inherited init;
        for i := 1 to max do
                for j := 1 to max do
                begin
                        New(rational_ptr(M[i, j]));
                        rational_ptr(M[i, j])^ := M0[i, j];
                end;
end;

procedure rational_matrix_obj.set_to_zero(var p: Pointer);
begin
        rational_ptr(p)^.num := 0;
        rational_ptr(p)^.den := 1;
end;

procedure lowest_terms(var x, y: LongInt);
var
        a, b, r: LongInt;
begin
        a := x;
        b := y;
        a := Abs(a);
        repeat
                r := a mod b;
                a := b;
                b := r;
        until r = 0;
        x := x div a;
        y := y div a;
end;

procedure rational_matrix_obj.scalar_sum(const p, q: Pointer; var r: Pointer);
var
        x, y, z: rational;
begin
        x := rational_ptr(p)^;
        y := rational_ptr(q)^;
        z.num := x.num * y.den + y.num * x.den;
        z.den := x.den * y.den;
        lowest_terms(z.num, z.den);
        if not Assigned(r) then
                New(rational_ptr(r));
        rational_ptr(r)^ := z;
end;

procedure rational_matrix_obj.scalar_product(const p, q: Pointer; var r: Pointer);
var
        x, y, z: rational;
begin
        x := rational_ptr(p)^;
        y := rational_ptr(q)^;
        z.num := x.num * y.num;
        z.den := x.den * y.den;
        lowest_terms(z.num, z.den);
        if not Assigned(r) then
                New(rational_ptr(r));
        rational_ptr(r)^ := z;
end;

procedure rational_matrix_obj.convert_to_string(p: Pointer; var tt: str30);
var
        t1, t2: String[11];
        j: Word;
begin
        Str(rational_ptr(p)^.num, t1);
        Str(rational_ptr(p)^.den, t2);
        tt := t1 + '/' + t2;
        for j := Length(t1) + Length(t2) to 12 do tt := ' ' + tt;
end;

destructor rational_matrix_obj.done;
var
        i, j: Word;
begin
        for i := 1 to max do
                for j := 1 to max do
                begin
                        Dispose(rational_ptr(M[i, j]));
                end;
        inherited done;
end;

constructor complex_matrix_obj.init(M0: complex_matrix);
var
        i, j: Word;
begin
        inherited init;
        for i := 1 to max do
                for j := 1 to max do
                begin
                        New(complex_ptr(M[i, j]));
                        complex_ptr(M[i, j])^ := M0[i, j];
                end;
end;

procedure complex_matrix_obj.set_to_zero(var p: Pointer);
begin
        complex_ptr(p)^.re := 0.0;
        complex_ptr(p)^.im := 0.0;
end;

procedure complex_matrix_obj.scalar_sum(const p, q: Pointer; var r: Pointer);
var
        x, y, z: complex;
begin
        x := complex_ptr(p)^;
        y := complex_ptr(q)^;
        z.re := x.re + y.re;
        z.im := x.im + y.im;
        if r = nil then
                New(complex_ptr(r));
        complex_ptr(r)^ := z;
end;

procedure complex_matrix_obj.scalar_product(const p, q: Pointer; var r: Pointer);
var
        x, y, z: complex;
begin
        x := complex_ptr(p)^;
        y := complex_ptr(q)^;
        z.re := x.re * y.re - x.im * y.im;
        z.im := x.re * y.im + x.im * y.re;
        if not Assigned(r) then
                New(complex_ptr(r));
        complex_ptr(r)^ := z;
end;

procedure complex_matrix_obj.convert_to_string(p: Pointer; var tt: str30);
var
        t1, t2: String[6];
begin
        Str(complex_ptr(p)^.re:6:4, t1);
        Str(complex_ptr(p)^.im:6:4, t2);
        tt := '  ' + t1 + ' + ' + t2 + 'i';
end;

destructor complex_matrix_obj.done;
var
        i, j: Word;
begin
        for i := 1 to max do
                for j := 1 to max do
                begin
                        Dispose(complex_ptr(M[i, j]));
                end;
        inherited done;
end;
end.
