{$r+}
unit matrices;

interface

uses
        Crt;

const
        max = 3;

type
        str10 = String[10];
        str30 = String[30];
        matrix = array[1..max, 1..max] of Pointer;
        rational = record
                num, den: LongInt;
        end;
        rational_ptr = ^rational;
        rational_matrix = array[1..max, 1..max] of rational;
        real_ptr = ^Real;
        real_matrix = array[1..max, 1..max] of Real;
        complex = record
                re, im: Real;
        end;
        complex_ptr = ^complex;
        complex_matrix = array[1..max, 1..max] of complex;

        matrix_obj = object
                M: matrix;
                constructor init;
                procedure set_to_zero(var p: Pointer); virtual;
                procedure scalar_sum(const p, q: Pointer; var r: Pointer); virtual;
                procedure scalar_product(const p, q: Pointer; var r: Pointer); virtual;
                procedure matrix_sum(const A, B: matrix);
                procedure matrix_product(const A, B: matrix);
                procedure convert_to_string(p: Pointer; var tt: str30); virtual;
                procedure print(name: str10);
                destructor done;
        end;

        real_matrix_obj = object(matrix_obj)
                constructor init(M0: real_matrix);
                procedure set_to_zero(var p: Pointer); virtual;
                procedure scalar_sum(const p, q: Pointer; var r: Pointer); virtual;
                procedure scalar_product(const p, q: Pointer; var r: Pointer); virtual;
                procedure convert_to_string(p: Pointer; var tt: str30); virtual;
                destructor done;
        end;

        rational_matrix_obj = object(matrix_obj)
                constructor init(M0: rational_matrix);
                procedure set_to_zero(var p: Pointer); virtual;
                procedure scalar_sum(const p, q: Pointer; var r: Pointer); virtual;
                procedure scalar_product(const p, q: Pointer; var r: Pointer); virtual;
                procedure convert_to_string(p: Pointer; var tt: str30); virtual;
                destructor done;
        end;

        complex_matrix_obj = object(matrix_obj)
                constructor init(M0: complex_matrix);
                procedure set_to_zero(var p: Pointer); virtual;
                procedure scalar_sum(const p, q: Pointer; var r: Pointer); virtual;
                procedure scalar_product(const p, q: Pointer; var r: Pointer); virtual;
                procedure convert_to_string(p: Pointer; var tt: str30); virtual;
                destructor done;
        end;

implementation

constructor matrix_obj.init;
begin
end;

procedure matrix_obj.set_to_zero(var p: Pointer);
begin
end;

procedure matrix_obj.scalar_sum(const p, q: Pointer; var r: Pointer);
begin
end;

procedure matrix_obj.scalar_product(const p, q: Pointer; var r: Pointer);
begin
end;

procedure matrix_obj.matrix_sum(const A, B: matrix);
var
        i, j: Word;
begin
        for i := 1 to max do
                for j := 1 to max do
                        scalar_sum(A[i, j], B[i, j], M[i, j]);
end;

procedure matrix_obj.matrix_product(const A, B: matrix);
var
        i, j, k: Word;
        prod, sum: Pointer;
begin
        GetMem(prod, SizeOf(M[1, 1]));
        for i := 1 to max do
                for k := 1 to max do
                begin
                        sum := M[i, k];
                        set_to_zero(sum);
                        for j := 1 to max do
                        begin
                                scalar_product(A[i, j], B[j, k], prod);
                                scalar_sum(sum, prod, sum);
                        end;
        end;
        FreeMem(prod, SizeOf(M[1, 1]));
end;

procedure matrix_obj.convert_to_string(p: Pointer; var tt: str30);
begin
end;

procedure matrix_obj.print(name: str10);
var
        i, j, k, hold_y: Word;
        temp: str30;
begin
        hold_y := WhereY;
        GotoXY(1, hold_y + (max + 1) div 2);
        Write(name, ' = ');
        k := Length(name) + 4;
        GotoXY(k, hold_y); Write('-');
        for i := 1 to max do
        begin
                GotoXY(k, hold_y + i);
                Write('|');
                for j := 1 to max do
                begin
                        convert_to_string(M[i, j], temp);
                        Write(temp);
                end;
                Write('|');
        end;
        GotoXY(k, max + hold_y + 1); Write('L');
        GotoXY(k + Length(temp) * max + 1, hold_y);
        Write('');
        GotoXY(k + Length(temp) * max + 1, max + hold_y + 1);
        WriteLn('-');
end;

destructor matrix_obj.done;
begin
end;

constructor real_matrix_obj.init(M0: real_matrix);
var
        i, j: Word;
begin
        inherited init;
        for i := 1 to max do
                for j := 1 to max do
                begin
                        New(real_ptr(M[i, j]));
                        real_ptr(M[i, j])^ := M0[i, j];
                end;
end;

procedure real_matrix_obj.set_to_zero(var p: Pointer);
begin
        real_ptr(p)^ := 0.0;
end;

procedure real_matrix_obj.scalar_sum(const p, q: Pointer; var r: Pointer);
begin
        if not Assigned(r) then
                New(real_ptr(r));
        real_ptr(r)^ := real_ptr(p)^ + real_ptr(q)^;
end;

procedure real_matrix_obj.scalar_product(const p, q: Pointer; var r: Pointer);
begin
        if not Assigned(r) then
                New(real_ptr(r));
        real_ptr(r)^ := real_ptr(p)^ * real_ptr(q)^;
end;

procedure real_matrix_obj.convert_to_string(p: Pointer; var tt: str30);
begin
        Str(real_ptr(p)^:9:4, tt);
end;

destructor real_matrix_obj.done;
var
        i, j: Word;
begin
        for i := 1 to max do
                for j := 1 to max do
                begin
                        Dispose(real_ptr(M[i, j]));
                end;
        inherited done;
end;

constructor rational_matrix_obj.init(M0: rational_matrix);
var
        i, j: Word;
begin
        inherited init;
        for i := 1 to max do
                for j := 1 to max do
                begin
                        New(rational_ptr(M[i, j]));
                        rational_ptr(M[i, j])^ := M0[i, j];
                end;
end;

procedure rational_matrix_obj.set_to_zero(var p: Pointer);
begin
        rational_ptr(p)^.num := 0;
        rational_ptr(p)^.den := 1;
end;

procedure lowest_terms(var x, y: LongInt);
var
        a, b, r: LongInt;
begin
        a := x;
        b := y;
        a := Abs(a);
        repeat
                r := a mod b;
                a := b;
                b := r;
        until r = 0;
        x := x div a;
        y := y div a;
end;

procedure rational_matrix_obj.scalar_sum(const p, q: Pointer; var r: Pointer);
var
        x, y, z: rational;
begin
        x := rational_ptr(p)^;
        y := rational_ptr(q)^;
        z.num := x.num * y.den + y.num * x.den;
        z.den := x.den * y.den;
        lowest_terms(z.num, z.den);
        if not Assigned(r) then
                New(rational_ptr(r));
        rational_ptr(r)^ := z;
end;

procedure rational_matrix_obj.scalar_product(const p, q: Pointer; var r: Pointer);
var
        x, y, z: rational;
begin
        x := rational_ptr(p)^;
        y := rational_ptr(q)^;
        z.num := x.num * y.num;
        z.den := x.den * y.den;
        lowest_terms(z.num, z.den);
        if not Assigned(r) then
                New(rational_ptr(r));
        rational_ptr(r)^ := z;
end;

procedure rational_matrix_obj.convert_to_string(p: Pointer; var tt: str30);
var
        t1, t2: String[11];
        j: Word;
begin
        Str(rational_ptr(p)^.num, t1);
        Str(rational_ptr(p)^.den, t2);
        tt := t1 + '/' + t2;
        for j := Length(t1) + Length(t2) to 12 do tt := ' ' + tt;
end;

destructor rational_matrix_obj.done;
var
        i, j: Word;
begin
        for i := 1 to max do
                for j := 1 to max do
                begin
                        Dispose(rational_ptr(M[i, j]));
                end;
        inherited done;
end;

constructor complex_matrix_obj.init(M0: complex_matrix);
var
        i, j: Word;
begin
        inherited init;
        for i := 1 to max do
                for j := 1 to max do
                begin
                        New(complex_ptr(M[i, j]));
                        complex_ptr(M[i, j])^ := M0[i, j];
                end;
end;

procedure complex_matrix_obj.set_to_zero(var p: Pointer);
begin
        complex_ptr(p)^.re := 0.0;
        complex_ptr(p)^.im := 0.0;
end;

procedure complex_matrix_obj.scalar_sum(const p, q: Pointer; var r: Pointer);
var
        x, y, z: complex;
begin
        x := complex_ptr(p)^;
        y := complex_ptr(q)^;
        z.re := x.re + y.re;
        z.im := x.im + y.im;
        if r = nil then
                New(complex_ptr(r));
        complex_ptr(r)^ := z;
end;

procedure complex_matrix_obj.scalar_product(const p, q: Pointer; var r: Pointer);
var
        x, y, z: complex;
begin
        x := complex_ptr(p)^;
        y := complex_ptr(q)^;
        z.re := x.re * y.re - x.im * y.im;
        z.im := x.re * y.im + x.im * y.re;
        if not Assigned(r) then
                New(complex_ptr(r));
        complex_ptr(r)^ := z;
end;

procedure complex_matrix_obj.convert_to_string(p: Pointer; var tt: str30);
var
        t1, t2: String[6];
begin
        Str(complex_ptr(p)^.re:6:4, t1);
        Str(complex_ptr(p)^.im:6:4, t2);
        tt := '  ' + t1 + ' + ' + t2 + 'i';
end;

destructor complex_matrix_obj.done;
var
        i, j: Word;
begin
        for i := 1 to max do
                for j := 1 to max do
                begin
                        Dispose(complex_ptr(M[i, j]));
                end;
        inherited done;
end;
end.
