implementation

uses
        Crt;

const
        eps = 1.0e-10;

var
        change: Boolean;
        no_vectors: Word;

{$ifdef complete_pivot}
var
        s, t: Word;
        column_rec: array[1..max_no_rows] of Word;
{$endif}

procedure correct_input(var A: matrix; first_col, last_col: Word);
...

procedure get_system_matrix(var A: matrix);
...

procedure get_rhs(var A: matrix);
...

procedure print_matrix(const A: matrix; name: str10;
                screen_row, first_col, last_col: Word);
...

procedure subtract_rows(var A: matrix; p, q: Word; x: Real);
var
        j: Word;
begin
        for j := p + 1 to no_cols do
                A[q, j] := A[q, j] - x * A[p, j];
end;

{$ifdef complete_pivot}
procedure change_columns(var A: matrix; p, t: Word);

var
        j: Word;
        x: Real;

begin
        for j := 1 to no_rows do
        begin
                x := A[j, p];
                A[j, p] := A[j, t];
                A[j, t] := x;
        end;
end;
{$endif}

procedure triangularize(var A: matrix);

var
        p, q, k: Word;

function partial_pivot(p: Word): Word;

var
        j, q: Word;
        x, y: Real;

begin
        q := p;
        x := Abs(A[p, p]);
        for j := p + 1 to no_rows do
        begin
                y := Abs(A[j, p]);
                if y > x then
                begin
                        q := j;
                        x := y
                end;
        end;
        singular := (Abs(x) < eps);
        partial_pivot := q;
end;{partial_pivot}

procedure change_rows(p, q: Word);

var
        j: Word;
        x: Real;

begin
        for j := p to no_cols do
        begin
                x := A[p, j];
                A[p, j] := A[q, j];
                A[q, j] := x;
        end
end;{change_rows}

{$ifdef complete_pivot}
procedure find_complete_pivot(p: Word; var s, t: Word);

var
        j, k: Word;
        x, y: Real;

begin
        s := p;
        t := p;
        x := Abs(A[p, p]);
        for j := p to no_rows do
                for k := p to no_rows do
                begin
                        y := Abs(A[j, k]);
                        if y > x then
                        begin
                                x := y;
                                s := j;
                                t := k;
                        end;
                end;
                singular := (Abs(x) < eps);
end;{find_complete_pivot}
{$endif}

begin{triangularize}
        p := 1;
        repeat
        {$ifdef complete_pivot}
                find_complete_pivot(p, s, t);
                if not singular then
                begin
                        if s > p then
                                change_rows(p, s);
                        if t > p then
                        begin
                                change_columns(A, p, t);
                                column_rec[p] := t;
                        end
                        else
                                column_rec[p] := p;
        {$else}
                k := partial_pivot(p);
                if not singular then
                begin
                        if k > p then
                                change_rows(p, k);
        {$endif}
                        for q := p + 1 to no_rows do
                        subtract_rows(A, p, q, A[q, p] / A[p, p]);
                end;
                Inc(p);
        until (p > no_rows) or singular;
end;{triangularize}

procedure back_subtract(var A: matrix);

var
        p, q: Word;

procedure normalize_row(p: Word);

var
        k: Word;
        x: Real;

begin
        x := A[p, p];
        A[p, p] := 1.0;
        for k := p + 1 to no_cols do
                A[p, k] := A[p, k] / x;
end;{normalize_row}

{$ifdef complete_pivot}
procedure restore_col_order;

var
        j, k: Word;
        x: Real;

begin
        for j := no_rows - 1 downto 1 do
        begin
                p := column_rec[J];
                if p > j then
                begin
                        for k := no_rows + 1 to no_cols do
                        begin
                                x := A[j, k];
                                A[j, k] := A[p, k];
                                A[p, k] := x;
                        end;
                end;
        end;
end;{restore_col_order}
{$endif}

begin{back_subtract}
        for p := 1 to no_rows do
                normalize_row(p);
        for p := no_rows downto 2 do
                for q := p - 1 downto 1 do
                        subtract_rows(A, p, q, A[q, p]);
{$ifdef complete_pivot}
        restore_col_order;
{$endif}
end;{back_subtract}

end.
