procedure QR_algorithm(var A: symmetric_matrix;
                var char_root: vector; var W: symmetric_matrix);

var
        diagonal, sine, cosine,
        super_diag, super_diag2: vector;
        norm, eps: Real;
        m, n: Word;
        A0: symmetric_matrix;
        c, s, x, x1, y, z, u, lambda, mu: Real;

procedure initialize_QR;

var
        j, k: Word;

procedure tri_diagonalize(var A, W: symmetric_matrix;
                var diagonal, super_diag: vector);

var
        c, t, x, y: Real;
        v: vector;
        i, j, k: Word;

procedure Householder_step(i: Word);

var
        p: vector;
        j, k: Word;
        z: Real;

begin
        for j := i + 1 to dim do
        begin
                t := 0.0;
                for k := i + 1 to dim do
                        t := t + A[j, k] * v[k];
                p[j] := 2.0 * t;
        end;
        z := 0.0;
        for j := i + 1 to dim do
                z := z + p[j] * v[j];
        z := 2.0 * z;
        for j := i + 1 to dim do
                for k := j to dim do
                        A[j, k] := A[j, k] - v[j] * p[k] - (p[j] - z * v[j]) * v[k];
end;{Householder_step}

procedure update_W(i: Word);

var
        p: vector;
        j, k: Word;

begin
        for j := 1 to dim do
        begin
                t := 0.0;
                for k := i + 1 to dim do
                        t := t + W[j, k] * v[k];
                p[j] := 2.0 * t;
        end;
        for j := 1 to dim do
                for k := i + 1 to dim do
                        W[j, k] := W[j, k] - p[j] * v[k];
end;{update_W}

begin{tri_diagonalize}
        super_diag[dim] := 0.0;
        for i := 1 to dim - 2 do
        begin
                diagonal[i] := A[i, i];
                t := 0.0;
                for j := i + 1 to dim do
                        t := t + Sqr(A[i, j]);
                c := Sqrt(t);
                x := A[i, i + 1];
                if x > 0.0 then
                        c := -c;
                super_diag[i] := c;
                if c <> 0.0 then
                begin
                        y := Sqrt(0.5 * (1.0 - x / c));
                        v[i + 1] := y;
                        y := -1.0 / (2.0 * c * y);
                        for j := i + 2 to dim do
                                v[j] := y * A[i, j];
                        Householder_step(i);
                        update_W(i);
                end;
        end;
        k := dim - 1;
        diagonal[k] := A[k, k];
        diagonal[dim] := A[dim, dim];
        super_diag[k] := A[k, dim];
        super_diag[dim] := 0.0;
end;{tri_diagonalize}

begin{initialize_QR}
        for j := 1 to dim do
        begin
                for k := 1 to dim do
                        W[j, k] := 0.0;
                W[j, j] := 1.0;
        end;
        tri_diagonalize(A, W, diagonal, super_diag);
        norm := 0.0;
        y := 0.0;
        for j := 1 to dim do
        begin
                x := y + Abs(diagonal[j]);
                y := Abs(super_diag[j]);
                x := x + y;
                if x > norm then
                        norm := x;
        end;
        eps := norm * tol;
        for j := 0 to dim do
        begin
                sine[j] := 0.0;
                cosine[j] := 0.0;
        end;
end;{initialize_QR}

procedure translate(a, b, c: Real; var lambda, mu: Real);

var
        trace, det, lambda0, b2, srt: Real;

begin
        b2 := Sqr(b);
        det := a * c - b2;
        trace := a + c;
        srt := Sqrt(sqr(trace) - 4.0 * det);
        if trace >= 0.0 then
                lambda := trace + srt
        else
                lambda := trace - srt;
        lambda := 0.5 * lambda;
        lambda0 := det / lambda;
        if Abs(lambda0 - mu) < 0.5 * Abs(lambda0) then
        begin
                lambda := lambda0;
                mu := lambda0
        end
        else
                if Abs(lambda - mu) < 0.5 * Abs(lambda) then
                        mu := lambda
                else
                begin
                        mu := lambda0;
                        lambda := 0.0;
                end;
end;{translate}

procedure factor(m, n: Word);

var
        j: Word;

begin
        diagonal[m] := diagonal[m] - lambda;
        u := super_diag[m];
        for j := m to n - 1 do
        begin
                x := diagonal[j];
                x1 := diagonal[j + 1] - lambda;
                y := super_diag[j];
                z := sqrt(Sqr(x) + Sqr(u));
                c := x / z;
                s := u / z;
                cosine[j] := c;
                sine[j]:= s;
                diagonal[j] := z;
                diagonal[j + 1] := -s * y + c * x1;
                super_diag[j] := c * y + s * x1;
                u := super_diag[j + 1];
                super_diag[j + 1] := c * u;
                super_diag2[j] := s * u;
        end;
end;{factor}

procedure multiply(m, n: Word);

var
        j, k: Word;

begin
        for k := m to n - 1 do
        begin
                c := cosine[k];
                s := sine[k];
                x := diagonal[k];
                y := super_diag[k];
                super_diag[k - 1] := super_diag[k - 1] * c +    super_diag2[k - 1] * s;
                diagonal[k] := x * c + y * s + lambda;
                super_diag[k] := -x * s + y * c;
                diagonal[k + 1] := diagonal[k + 1] * c;
                for j := 1 to dim do
                begin
                        x := W[j, k];
                        y := W[j, k + 1];
                        W[j, k] := x * c + y * s;
                        W[j, k + 1] := -x * s + y * c;
                end;
        end;
        diagonal[n] := diagonal[n] + lambda;
end;{multiply}

procedure report_result;
...
begin{QR_algorithm}
        A0 := A;
        initialize_QR;
        n := dim;
        mu := 0.0;
        while n > 0 do
        begin
                if Abs(super_diag[n - 1]) < eps then
                begin
                        char_root[n] := diagonal[n];
                        Dec(n);
                end
                else
                begin
                        m := n - 1;
                        repeat
                                Dec(m)
                        until Abs(super_diag[m]) < eps;
                        Inc(m);
                        translate(diagonal[n - 1], super_diag[n - 1],
                                diagonal[n], lambda, mu);
                        factor(m, n);
                        super_diag2[m - 1] := 0.0;
                        super_diag[m - 1] := 0.0;
                        multiply(m, n);
                end;
        end;
end;{QR_algorithm}

end.
