program nonlinear_system;

uses
        Crt;

const
        max = 20;
        eps = 1.0e-10;
        eta = 1.0e-3;

type
        vector_fun = function(j: Word; x, y: Real): Real;

var
        x, y: Real;
        i, n: Word;
        ch: Char;

function F(x, y: Real): Real;
begin
        F := Cos(x) * Cos(y) - 0.1 * x - 0.2 * y + 0.15 * x * y;
end;

function G(j: Word; x, y: Real): Real; far;
begin
        if j = 1 then
                G := -Sin(x) * Cos(y) - 0.1 + 0.15 * y
        else
                G := -Cos(x) * Sin(y) - 0.2 + 0.15 * x;
end;

function dG(j, k: Word; x, y: Real): Real; far;
begin
        if j = 1 then
        begin
                if k = 1 then
                        dG := -Cos(x) * Cos(y)
                else
                        dG := Sin(x) * Sin(y) + 0.15;
        end
        else
        begin
                if k = 1 then
                        dG := Sin(x) * Sin(y) + 0.15
                else
                        dG := -Cos(x) * Cos(y);
        end;
end;

procedure solve(G: vector_fun; var x, y: Real);

var
        j, k: Integer;
        r, s, t, u, v: Real;
        singular: Boolean;
        ch: Char;

procedure Newton_step(var x, y: Real);

var
        det: Real;
        j, k: Word;

begin
        u := G(1, x, y);
        v := G(2, x, y);
        r := dG(1, 1, x, y);
        s := dG(1, 2, x, y);
        t := dG(2, 2, x, y);
        det := r * t - s * s;
        if Abs(det) < eta then
                singular := True
        else
        begin
                singular := False;
                x := x - (t * u - s * v) / det;
                y := y - (-s * u + r * v) / det;
        end
end;{Newton_step}

begin{solve}
        repeat
                Newton_step(x, y);
                Dec(i);
        until singular or (i = 0) or (Abs(u) + Abs(v) < eps);
        if singular then
                WriteLn('襭 ')
        else
                if i = 0 then
                begin
                        WriteLn(n, ' 権');
                        WriteLn('F(', x, ', ', y, ') = (', u, ', ', v, ')')
                end
                else
                        WriteLn('襭: (x, y) = (', x, ', ', y, ')');
end;{solve}

begin
        ClrScr;
        x := 0.0;
        y := 0.0;
        i := max;
        solve(G, x, y);
        WriteLn('F(x, y) =', F(x, y));
        WriteLn;
        Write(' <Enter>'); ReadLn;
end.
