function adaptive_simpson(F: real_fun; x0, x1, eps, eta: Real): Real;

const
        max_level = 35;

var
        k, nest_level: Word;
        integral_abs: Real;

function simpson3point(x0, delta_x, estimate, integral_abs,
                eps, eta, left, middle, right: Real): Real;

var
        dx3, sum, eps3, eta3, factor, left_integ,
        middle_integ, right_integ, F1, F2, F4, F5: Real;

begin
        Inc(nest_level);
        dx3 := delta_x / 3.0;
        F1 := F(x0 + 0.5 * dx3);
        F2 := F(x0 + dx3);
        F4 := F(x0 + 2.0 * dx3);
        F5 := F(x0 + 2.5 * dx3);
        Inc(no_evaluations, 4);
        factor := dx3 / 6.0;
        left_integ := factor * (left + 4.0 * F1 + F2);
        middle_integ := factor * (F2 + 4.0 * middle + F4);
        right_integ := factor * (F4 + 4.0 * F5 + right);
        sum := left_integ + middle_integ + right_integ;
        integral_abs := integral_abs - Abs(estimate)
                + Abs(left_integ) + Abs(middle_integ) + Abs(right_integ);
        if (nest_level > 1 ) and ((nest_level = max_level) or
                (Abs(sum - estimate) <= eps + eta * integral_abs)) then
                        simpson3point := sum
        else
        begin
                if nest_level > highest_level then
                        Inc(highest_level);
                eps3 := 0.577 * eps;
                eta3 := 0.577 * eta;
                left_integ := simpson3point(x0, dx3, left_integ, integral_abs, eps3, eta3, left, F1, F2);
                middle_integ := simpson3point(x0 + dx3, dx3,
                        middle_integ, integral_abs, eps3, eta3, F2, middle, F4);
                right_integ := simpson3point(x0 + 2.0 * dx3, dx3,
                        right_integ, integral_abs, eps3, eta3, F4, F5, right);
                simpson3point := left_integ + middle_integ + right_integ;
        end;
        Dec(nest_level);
end;{simpson3point}

begin{adaptive_simpson}
        nest_level := 1;
        highest_level := 1;
        no_evaluations := 3;
        adaptive_simpson := simpson3point(x0, x1 - x0, 0.0, 0.0, eps, eta, F(x0), F(x0 + 0.5 * (x1 - x0)), F(x1));
end;{adaptive_simpson}
