function romberg(F: real_fun; x0, x1, eps, eta: Real; min, max: Word): Real;

const
        abs_max = 30;

var
        p, dx, error, F_of_x0, F_of_x1, F_of_xk,
        roundoff_error, integral_abs, tolerance,
        previous_estimate, current_estimate,
        mid_sum, temp_sum, mid_sum_abs: Real;
        table: array[0..abs_max] of Real;
        j, n: Word;
        k, r: LongInt;
        done: Boolean;
        denom: array[1..abs_max] of Real;

begin
        p := 1.0;
        for k := 1 to abs_max do
        begin
                p := 4.0 * p;
                denom[k] := 1.0 / (p - 1.0);
        end;
        dx := x1 - x0;
        F_of_x0 := F(x0);
        F_of_x1 := F(x1);
        current_estimate := 0.0;
        previous_estimate := 0.0;
        done := False;
        table[0] := 0.5 * dx * (F_of_x0 + F_of_x1);
        integral_abs := 0.5 * Abs(dx) * (Abs(F_of_x0) + Abs(F_of_x1));
        n := 1;
        r := 1;
        repeat
                dx := 0.5 * dx;
                mid_sum := 0.0;
                mid_sum_abs := 0.0;
                roundoff_error := 0.0;
                for k := 1 to r do
                begin
                        F_of_xk := F(x0 + (2 * k - 1) * dx);
                        mid_sum_abs := mid_sum_abs + Abs(F_of_xk);
                        F_of_xk := F_of_xk + roundoff_error;
                        temp_sum := mid_sum + F_of_xk;
                        roundoff_error := (mid_sum - temp_sum) + F_of_xk;
                        mid_sum := temp_sum;
                        if KeyPressed then
                                Halt;
                end;
                table[n] := 0.5 * table[n - 1] + dx * mid_sum;
                integral_abs := 0.5 * integral_abs + Abs(dx) * mid_sum_abs;
                for j := n - 1 downto 0 do
                        table[j] := table[j + 1] + denom[n - j]*(table[j + 1] - table[j]);
                if n >= min then
                begin
                        tolerance := eta * integral_abs + eps;
                        error := Abs(table[0] - current_estimate)
                                + Abs(current_estimate - previous_estimate);
                        done := (error < tolerance);
                end;
                Inc(n);
                done := done or (n > max);
                previous_estimate := current_estimate;
                current_estimate := table[0];
                r := r + r;
        until done;
        romberg := current_estimate;
end;
